clear variables;close all;clc
[X,Xt] = xlsread('Study1input\Study1_text.xls'); %#ok<XLSRD> % loads raw data (992 participants, 9 columns)
% X is a 992 x 9 matrix of numbers
% Xt is a corresponding 992 x 9 matrix of texts
%% Submit batches to ChatGPT API
import matlab.net.* matlab.net.http.* matlab.net.http.io.*
key % read the private key
header = [HeaderField('Content-Type', 'application/json')
          HeaderField('Authorization', api_key)]';
uri = URI('https://api.openai.com/v1/chat/completions');

%prompt_no = 1; % The comments below were obtained from respondents in an online survey about ...
 prompt_no = 2; % Looking at the comments, score the interface, from 1 to 100. Only report a number between 1 and 100, rounded to two decimals (r = 0.993)
%prompt_no = 3; % Looking at the comments, score the interface, from 1 to 100. Only report a number between 1 and 100, rounded to two decimals - batches of 25 (r = 0.987) 
%prompt_no = 4; % Looking at the comments, score the interface, from 1 to 100. Only report a number between 1 and 100, rounded to two decimals - one large batch (r = 0.724) (16 k)

if ismember(prompt_no,[1 2])
    si=1:100:1000; % start index
    ei=si+99; % end index
    ei(end)=size(Xt,1);
elseif prompt_no==3
    si=1:25:1000; % start index
    ei=si+24; % end index
    ei(end)=size(Xt,1);
elseif prompt_no==4
    si=1;
    ei=992;
end

run_api=1;

if run_api==1 % if this flag equals 1, then loop through the batches as described in the paper

    for j=1:9 % loop over 9 interfaces
        for i=1:length(si)
            a = Xt(si(i):ei(i),j); % all comments of batch
            text=[];
            for i2=1:size(a,1) % loop over individual comments
                         line=char(a(i2));
                if ~isempty(line)
                    text=[text line newline]; %#ok<AGROW>
                end
            end

            if prompt_no==1
                prompt = ['The comments below were obtained from respondents in an online survey about an AR interface in a road-crossing scenario. If you read the comments below, what is the quality of the interface on a scale from 1 = bad to 10 = good? Only report a number between 1 and 10. Round to two decimals' newline];
            elseif ismember(prompt_no,2:4)
                prompt = ['Looking at the comments, score the interface, from 1 to 100. Only report a number between 1 and 100, rounded to two decimals' newline];
            end
            prompt=[prompt text]; %#ok<AGROW>
            if ismember(prompt_no,1:3)
                body = JSONProvider(struct('model','gpt-3.5-turbo','temperature',0,'n',1,'messages', {{struct('role', 'user','content', prompt')}}));
            else
                body = JSONProvider(struct('model','gpt-3.5-turbo-16k','temperature',0,'n',1,'messages', {{struct('role', 'user','content', prompt')}}));
            end
            response = RequestMessage('post', header, body).send(uri.EncodedURI);
            if prompt_no==1 || prompt_no==3
                save(['Response_AR_4_mar_2023_4_992_' num2str(j) '_comments_' num2str(si(i)) '_to_' num2str(ei(i))],'response','prompt')
            end
            if prompt_no==2
                save(['Response_AR_7_mar_2023_4_992_' num2str(j) '_comments_' num2str(si(i)) '_to_' num2str(ei(i))],'response','prompt')
            end
            if prompt_no==4
                save(['Response_AR_July_14_16k_992_' num2str(j) '_comments_' num2str(si(i)) '_to_' num2str(ei(i))],'response','prompt')
            end
            try
                disp(['Batch' num2str(si(i)) '_to_' num2str(ei(i))])
                response_text = response.Body.Data.choices(1).message.content;
                disp(response_text);
            catch error
                pause
            end

        end
    end
end
%% Extract scores from ChatGPT outputs

Score=NaN(9,length(si),1);
for j=1:9 % loop over 9 interfaces
    for i=1:length(si) % loop over batches
        if prompt_no==1
            load(['Study1output\Response_AR_4_mar_2023_4_992_' num2str(j) '_comments_' num2str(si(i)) '_to_' num2str(ei(i))])
        elseif prompt_no==2
            load(['Study1output\Response_AR_7_mar_2023_4_992_' num2str(j) '_comments_' num2str(si(i)) '_to_' num2str(ei(i))])
        elseif prompt_no==3
            load(['Study1output_batchesof25\Response_AR_4_mar_2023_4_992_' num2str(j) '_comments_' num2str(si(i)) '_to_' num2str(ei(i))])
        elseif prompt_no==4
            load(['Study1output_16k\Response_AR_July_14_16k_992_' num2str(j) '_comments_' num2str(si(i)) '_to_' num2str(ei(i))])
        end
        if isequal(response.StatusCode,'OK')
            n=length(response.Body.Data.choices);
            for k=1:n
                response_text = response.Body.Data.choices(k).message.content;
                response_text = response_text(ismember(response_text,[46 48:57])); % only retain numbers https://en.wikipedia.org/wiki/List_of_Unicode_characters

                dot=find(response_text==46); % index of the dot
                if length(dot)>1
                    dot = dot(2:end);
                    response_text(dot) = []; % throw away extra dots
                end

                Score(j,i,k)=str2double(response_text);
                if Score(j,i,k)>100
                    Score(j,i,k)=NaN;
                end
                if isnan(Score(j,i,k))
                    disp([j i k])
                    disp(response_text)
                end % end of loop of reps
            end % end of statuscode ok
        end
    end
end
%% Create scatter plot figure
close all
v1=mean(X)'; % composite score, as reported in Tabone et al. (2023)
v2=mean(Score,2); % ChatGPT sentiment score
documents = tokenizedDocument(Xt);
v3 = mean(vaderSentimentScores(documents))';

disp(['Correlation coefficient between ChatGPT sentiment score and composite score: ' num2str(corr(v2,v1))])
disp(['Correlation coefficient between ChatGPT sentiment score and VADER sentiment score: ' num2str(corr(v2,v3))]) 
disp(['Correlation coefficient between VADER sentiment score and composite score: ' num2str(corr(v3,v1))]) 

interfaces={'Augmented zebra crossing'
    'Planes on vehicle'
    'Conspicious looming planes'
    'Field of safe travel'
    'Fixed pedestrian lights'
    'Virtual fence'
    'Phantom car'
    'Nudge HUD'
    'Pedestrian lights HUD'};

cm=[                     0                         0                         1 % matrix of colors
                         1                         0                         0
                         0                         1                         0
                         0                         0         0.172413793103448
                         1         0.103448275862069         0.724137931034483
                         1         0.827586206896552                         0
                         0         0.344827586206897                         0
         0.517241379310345         0.517241379310345                         1
         0.620689655172414         0.310344827586207         0.275862068965517];

clear text

figure
for i = 1:9 % loop over 9 interfaces
    scatter1 = scatter(v2(i),v1(i), 400, 'markerfacecolor', cm(i, :),  'markeredgecolor', cm(i, :));
    hold on;
    grid on;
    scatter1.MarkerFaceAlpha = 0.8;
    text(v2(i) + 0.45, v1(i), char(interfaces(i)), 'horizontalalignment', 'left', 'fontsize', 24)
end
set(gca,'fontsize', 24)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])
axis square
xlabel('Sentiment score from ChatGPT''s text analysis (1-100)','fontsize', 24)
ylabel('Composite score of human responses to rating scales (\it{z}\rm-score)','fontsize', 24)

figure
for i = 1:9 % loop over 9 interfaces
    scatter1 = scatter(v2(i),v3(i), 400, 'markerfacecolor', cm(i, :),  'markeredgecolor', cm(i, :));
    hold on;
    grid on;
    scatter1.MarkerFaceAlpha = 0.8;
    text(v2(i) + 0.45, v3(i), char(interfaces(i)), 'horizontalalignment', 'left', 'fontsize', 24)
end
set(gca,'fontsize', 24)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01])
axis square
xlabel('Sentiment score from ChatGPT''s text analysis (1-100)','fontsize', 24)
ylabel('VADER sentiment score (-1 to 1)','fontsize', 24)
