clear variables;close all;clc
import matlab.net.* matlab.net.http.* matlab.net.http.io.*
uri = URI('https://api.openai.com/v1/chat/completions');

key % read the private key

compare_pairs = 0;
create_summaries = 0;
create_final_summary = 1;

%% 
% Here, we obtain "two sets of 567 descriptions of how the transcripts for replica and real The Night Watch differed from 
% one another, once for the situation where the real painting was first mentioned in the prompt, and 
% another time where the replica was first mentioned in the prompt

if compare_pairs==1
    [~,Real]=xlsread('Study3input\Transcripts_real.xlsx'); %#ok<*XLSRD>
    [~,Replica]=xlsread('Study3input\Transcripts_replica.xlsx');

    % Process transcripts for Real painting
    RealTranscripts=Real(2:end,2);
    RealParticipantIndices=find(contains(Real(2:end,1),'Participant'));
    for i=1:length(RealParticipantIndices)
        if i<length(RealParticipantIndices)
            ParticipantNumberReal(RealParticipantIndices(i):RealParticipantIndices(i+1))=i;
        else
            ParticipantNumberReal(RealParticipantIndices(i):length(RealTranscripts))=i;
        end
    end

    % Process transcripts for Replica painting
    ReplicaTranscripts=Replica(2:end,2);
    ReplicaParticipantIndices=find(contains(Replica(2:end,1),'Participant'));
    for i=1:length(ReplicaParticipantIndices)
        if i<length(ReplicaParticipantIndices)
            ParticipantNumberReplica(ReplicaParticipantIndices(i):ReplicaParticipantIndices(i+1))=i;
        else
            ParticipantNumberReplica(ReplicaParticipantIndices(i):length(ReplicaTranscripts))=i;
        end
    end

    [a,b]=ndgrid(1:27, 1:21);Pairs=[a(:) b(:)]; % 21 participants for the real painting, 27 participants for the replica painting
    rng(1);
    [~,order]=sort(rand(size(Pairs,1),1));
    Pairs=Pairs(order,:); % sort all pairs in random order
    for r=1:2
        if r==1 % real followed by replica
            diary off
            %           delete('RealReplica.txt')
            diary RealReplica.txt
        end
        if r==2 % replica followed by real
            diary off
            %            delete('ReplicaReal.txt')
            diary ReplicaReal.txt
        end

        for p = 1:size(Pairs,1) % loop over all 567 pairs
            replica_transcript = strjoin(ReplicaTranscripts(ParticipantNumberReplica==Pairs(p,1)));
            real_transcript = strjoin(RealTranscripts(ParticipantNumberReal==Pairs(p,2)));

            if r==1
                prompt = ['TRANSCRIPT 1:' newline real_transcript newline newline 'TRANSCRIPT 2:' newline replica_transcript newline newline 'What would you regard as the key difference between both transcripts?'];
            elseif r==2
                prompt = ['TRANSCRIPT 1:' newline replica_transcript newline newline 'TRANSCRIPT 2:' newline real_transcript newline newline 'What would you regard as the key difference between both transcripts?'];
            end

            error=1;

            while error==1
                try
                    body = JSONProvider(struct('model','gpt-3.5-turbo-0613','temperature',0,'n',1,'messages', {{struct('role', 'user','content', prompt')}}));
                    response = RequestMessage('post', [HeaderField('Content-Type', 'application/json') HeaderField('Authorization', api_key)], body).send(uri.EncodedURI);
                    disp(['Participant pair' num2str(p)])
                    disp(response.Body.Data.choices.message.content)
                    error=0;
                catch error
                end
            end

        end % end of loop over Pairs
    end % end of: for r=1:2
    diary off
end % end of: compare_pairs ==1

%% 
% "Subsequently, we implemented the following procedure for each sequence to obtain a final summary. 
% First, we separated the 567 pairwise comparisons into 7 batches of 81 each, maintaining their original 
% random order. We then submitted each batch to the ChatGPT API (model: gpt-3.5-turbo-16k) using 
% the following prompt: "These are transcripts of different pairs of speakers. Provide one single 200-
% word summary of all Transcripts 1 combined; as well as one single 200-word summary of all Transcripts 2 
% combined. Report as follows: "Transcript 1: ...; Transcript 2: …”".

if create_summaries==1
    import matlab.net.* matlab.net.http.* matlab.net.http.io.*
    uri = URI('https://api.openai.com/v1/chat/completions');
    key
    si=1:81:567; % start index of pairs
    for r=1:2
        if r==1 % real followed by replica
            diary off
            disp('Summaries - Real followed by replica')
            try
                %                delete('SummariesRealReplica.txt')
            catch error
            end
            diary SummariesRealReplica.txt
            all_paired_comparisons = fileread('Study3output\RealReplica.txt');
        end
        if r==2 % replica followed by real
            diary off
            disp('Summaries - Replica followed by real')
            try
                %      delete('SummariesReplicaReal.txt')
            catch error
            end
            diary SummariesReplicaReal.txt
            all_paired_comparisons = fileread('Study3output\ReplicaReal.txt');
        end
        for v=1:length(si) % loop over batches
            disp(['BATCH ' num2str(v)])
            si_pair=strfind(all_paired_comparisons,['Participant pair' num2str(si(v)) newline]);
            if v==length(si) % last batch
                ei_pair=length(all_paired_comparisons);
            else
                ei_pair=strfind(all_paired_comparisons,['Participant pair' num2str(si(v+1)) newline])-2;
            end
            prompt=['These are transcripts of different pairs of speakers. Provide one single 200-word summary of all Transcripts 1 combined; as well as one single 200-word summary of all Transcripts 2 combined. Report as follows: "Transcript 1: ...;  Transcript 2: ..."' newline newline all_paired_comparisons(si_pair:ei_pair)];
            error=1;
            while error==1
                try
                    body = JSONProvider(struct('model','gpt-3.5-turbo-16k','temperature',0,'n',1,'messages', {{struct('role', 'user','content', prompt')}}));
                    response = RequestMessage('post', [HeaderField('Content-Type', 'application/json') HeaderField('Authorization', api_key)], body).send(uri.EncodedURI);
                    disp(response.Body.Data.choices.message.content)
                    error=0;
                catch error
                end
            end
        end
    end
    diary off
end
diary off
%%
% "Lastly, for both sequences, we executed a prompt to create a final summary, using the following prompt: 
% 'Provide one single 200-word summary of the differences between Transcripts 1 and 2, of all batches combined:", followed by the summaries of the 7 batches.

if create_final_summary==1
    import matlab.net.* matlab.net.http.* matlab.net.http.io.*
    uri = URI('https://api.openai.com/v1/chat/completions');
    key
    for r=1:2
        if r==1 % real followed by replica
            disp('Final summary - Real followed by replica')
            all_summaries = fileread('Study3output\SummariesRealReplica.txt');
        end
        if r==2 % replica followed by real
            disp('Final summary - Replica followed by real')
            all_summaries = fileread('Study3output\SummariesReplicaReal.txt');
        end
        pause
        prompt=['Provide one single 200-word summary of the differences between Transcripts 1 and 2, of all batches combined:' newline newline all_summaries];
        body = JSONProvider(struct('model','gpt-3.5-turbo-0613','temperature',0,'n',1,'messages', {{struct('role', 'user','content', prompt')}}));
        response = RequestMessage('post', [HeaderField('Content-Type', 'application/json') HeaderField('Authorization', api_key)], body).send(uri.EncodedURI);
        disp(response.Body.Data.choices.message.content)
    end
end
